***************
*
*       68HC11 BOOTLOADER PROGRAM
*
*       THIS MONITOR PROGRAM IS INTENDED TO BE LOADED
*       BY THE BOOTLOAD PROCEEDURE IN BOOTSTRAP MODE.
*       THE PROGRAM PERMITS EDITING EPROM IN THE E9 VERSION, 
*       AND EEPROM IN ANY 68HC11 MICROCONTROLLER EXCEPT 
*       THE E20 VERSION.  NOTE THAT THE USER CAN MOVE FROM 
*       THE BOOTSTRAP MODE TO ANY OTHER MODE BY SOFTWARE, WITHOUT
*       A SYSTEM RESET.
*
*       THIS PROGRAM IS HARD ENCODED INTO THE PROGRAM 6811.C,
*       THE PC MONITOR PROGRAM, WHICH WILL AUTOMATICALLY LOAD
*       AND RUN IT, AS WELL AS SWITCH MODES IF DESIRED.
*
*       BY:             PHILIP DEVRIES
*       VERSION:        1.0 
*       DATE:           DECEMBER 1, 1994
*       FILENAME:       6811BOOT.ASM
*       COMPILED BY:    MOTOROLA FREEWARE AS11 CROSS-ASSEMBLER
*       
*       MEMORY USE     
*       BY PROGRAM:     0000 - 00FF
*       VECTOR TABLE:   BFD6 - BFFF (not used, and not initialized)
*       CONTROLLER
*       REGISTER LOC:   1000 - 103F
*
*       FOR USE WIT
*       PC MONITOR
*       VERSION:        3.X
*       FILENAME:       6811.C
*       COMPILED BY:    QUICK-C VERSION 2 BY MICROSOFT
*       
***************

***************
*       Equates   
***************

REGBS   EQU     $1000           Page location of registers
TCNT    EQU     REGBS+$0E       Timer counter
TOC5    EQU     REGBS+$1E       OC5 Timer
TMSK1   EQU     REGBS+$22       Timer mask 1
TFLG1   EQU     REGBS+$23       Timer interrupt flags 1
TMSK2   EQU     REGBS+$24       Timer mask 2
TFLG2   EQU     REGBS+$25       Timer interrupt flags 2
BAUD    EQU     REGBS+$2B       SCI baud reg
SCCR1   EQU     REGBS+$2C       SCI control1 reg
SCCR2   EQU     REGBS+$2D       SCI control2 reg
SCSR    EQU     REGBS+$2E       SCI status reg
SCDR    EQU     REGBS+$2F       SCI data reg
BPROT   EQU     REGBS+$35       Block protect
OPTION  EQU     REGBS+$39       Configuration options
COPRST  EQU     REGBS+$3A       COP reset register
PPROG   EQU     REGBS+$3B       EEPROM programming register
HIPRO   EQU     REGBS+$3C       Mode select
INIT    EQU     $103D           Memory map register
CONFIG  EQU     $103F           Startup configuration

STACK   EQU     $001F

***************
*       This is the reset location for all resets, cold boot, hard reset,
*       clock monitor, and COP.  Note that interrupts are disabled
*       (including XIRQ) on reset.
***************

        ORG     $0000
        

        LDS     #$00FF

        LDAB    #%00110000      Set the baud rate 9600
        STAB    BAUD
        
        LDAB    #%11000000      Set up number of bits serial to 8
        STAB    SCCR1
        
        LDAB    #%00101100      Enable SCI interupts,
        STAB    SCCR2           enable transmit and receive
        
        BRA     MAIN

**************
*       This subroutine waits for an input on the SCI (RS-232)
*       and returns the data in register A.  The COP timer is
*       also reset here.
**************

INCHAR  LDAA    #%01010101
        STAA    COPRST 
        LDAA    #%10101010
        STAA    COPRST
        LDAA    SCSR            Status of SCI
        ANDA    #%00100000      Pick out bit 5
        BEQ     INCHAR          If not set, there was no input
        LDAA    SCDR            If is set, load data and return
        RTS

**************
*       This subroutine waits for the transmitter buffer to
*       empty, then transmits the character from A
**************

OUTCHAR LDAB    SCSR    
        ANDB    #%11000000
        BEQ     OUTCHAR
        STAA    SCDR
        RTS

**************
*       This is the main loop of the monitor program
**************

MAIN    BSR     INCHAR          Get instruction code from pc monitor

        CMPA    #'S             S means send X to pc monitor
        BEQ     SENDADD
        
        CMPA    #'A             A means get X from pc monitor
        BEQ     GETADDR
        
        CMPA    #'D             D means send data to pc monitor
        BEQ     DUMPMEM         without increment
        
        CMPA    #'E             E means edit memory contents
        BEQ     EDITMEM         without increment
        
        CMPA    #'M             M means send data to pc monitor
        BEQ     DUMPIMEM        with increment

        CMPA    #'C             C means change memory contents
        BEQ     EDITIMEM        with increment
        
        CMPA    #'P             P means program eeprom
        BEQ     EEPROM

        CMPA    #'O             O means program eprom
        BEQ     EPROM
        
        CMPA    #'R             R means run monitor at BF00
        BNE     MAIN
        
        JMP     $BF00

**************
*       Send data from memory location X to pc monitor.
**************

DUMPMEM LDAA    ,X              Read data
        BSR     OUTCHAR         Send to pc monitor
        BRA     MAIN            

**************
*       Send data from memory location X to pc monitor.
*       Increment address pointer
**************

DUMPIMEM 
        LDAA    ,X              Read data
        BSR     OUTCHAR         Send to pc monitor
        INX
        BRA     MAIN            

**************
*       Receive data from pc monitor, save at memory location X. 
**************
                                           
EDITMEM BSR     INCHAR          Get data from pc monitor
        STAA    ,X              Store pc monitor data in memory
        BRA     MAIN            command C

**************
*       Receive data from pc monitor, save at memory location X. 
*       Increment address pointer
**************
                                           
EDITIMEM 
        BSR     INCHAR          Get data from pc monitor
        STAA    ,X              Store pc monitor data in memory
        INX
        BRA     MAIN            command C

**************
*       Receive address from pc monitor, save in X.
**************

GETADDR BSR     INCHAR          Get 1st character of address (LSBS)
        TAB                     Put in B, low address
        BSR     INCHAR          Get 2nd character of address (MSBS)
*                               Put in A, high address        
        XGDX                    Store address from D in X
        BRA     MAIN  

**************
*       Send the current address in X to the pc monitor.
**************

SENDADD XGDX                    Put X into D
        STAB    SCDR            Send LSB's
        BSR     OUTCHAR         Send MSB's
        XGDX                    Store address from D in X
        BRA     MAIN

**************
*       EPROM programming for 68HC711E9
**************
EPROM
        BSR     INCHAR          This is the data
        LDAB    #$20
        STAB    PPROG           Enable eprom latches
        STAA    ,X              Store data to address
        LDAB    #$21            
        STAB    PPROG           Enable eeprom programming voltage 
        LDY     #1142
        BSR     DELAY           Delay 4mS
        CLR     PPROG           Turn off programmer
        
        LDAA    ,X
        STAA    SCDR            Send data to monitor
        
        BRA     MAIN

**************
*       Routine to program a byte of memory at X
**************

EEPROM 
        JSR     INCHAR          This is the data
        
        LDAB    #$16            Control bits to indicate erase byte
        STAB    PPROG           Enable programming - disable EEPROM
        STAA    ,X              Write random data to address
        LDAB    #$17            
        STAB    PPROG           Start programmer
        LDY     #2857           
        BSR     DELAY           Wait 10mS
        CLR     PPROG
        
        LDAB    #$02            Control bits to indicate write byte
        STAB    PPROG           Enable programming - disable EEPROM
        STAA    ,X              Send data to EEPROM
        LDAB    #$03
        STAB    PPROG           Start programmer
        LDY     #2857           
        BSR     DELAY           Wait 10mS
        CLR     PPROG
        
        LDAA    ,X              Read data from location X
        STAA    SCDR            Send data back to monitor.  This forces
*                               the monitor to wait for programmer to
*                               finish
        INX                     
        JMP     MAIN

**************
*       Delay loop
**************

DELAY   DEY                     
        BNE     DELAY
        RTS

        ORG     $00E0

        FCC     "STACK STACK STACK STACK STACK ST"
